@@ -60,7 +60,7 @@ extern "C" {
 /* PLVERSION_TAG: a string, normally "", but for example "rc1" */
 
 #ifndef PLVERSION
-#define PLVERSION 90303
+#define PLVERSION 90305
 #endif
 #ifndef PLVERSION_TAG
 #define PLVERSION_TAG ""
@@ -188,12 +188,12 @@ be relied upon to remain unchanged across versions.
 		 *	       TYPES		*
 		 *******************************/
 
-typedef uintptr_t	_PLQ(word);	/* Anonymous ptr-sized object*/
+typedef uint64_t	_PLQ(word);	/* Anonymous Prolog data cell */
 #ifndef PL_HAVE_ATOM_T
 #define PL_HAVE_ATOM_T
 typedef uintptr_t	atom_t;		/* Prolog atom */
 #endif
-typedef _PLQ(word)	functor_t;	/* Name/arity pair */
+typedef uintptr_t	functor_t;	/* Name/arity pair */
 typedef uintptr_t	_PLQ(code);	/* Prolog bytecode type */
 typedef _PLS(module) *	module_t;	/* Prolog module */
 typedef _PLS(procedure) *predicate_t;	/* Prolog procedure */
@@ -206,7 +206,7 @@ typedef _PLS(queryRef) *qid_t;		/* opaque query handle */
 typedef uintptr_t	PL_fid_t;	/* opaque foreign context handle */
 typedef _PLS(foreign_context) *control_t; /* non-deterministic control arg */
 typedef _PLS(PL_local_data) *PL_engine_t; /* opaque engine handle */
-typedef uintptr_t	PL_atomic_t;	/* same a word */
+typedef _PLQ(word)	PL_atomic_t;	/* same a word */
 typedef uintptr_t	foreign_t;	/* return type of foreign functions */
 typedef wchar_t		pl_wchar_t;	/* Prolog wide character */
 #ifdef __cplusplus
@@ -292,6 +292,7 @@ typedef union
 #define PL_NOT_A_LIST	 (43)		/* Object is not a list */
 					/* dicts */
 #define PL_DICT		 (44)
+#define PL_SWORD	 (45)		/* Signed word (internal) */
 
 /* Or'ed flags for PL_set_prolog_flag() */
 /* MUST fit in a short int! */
@@ -497,6 +498,8 @@ PL_EXPORT(int)		PL_atom_mbchars(atom_t a, size_t *len, char **s,
 PL_EXPORT(const wchar_t *)	PL_atom_wchars(atom_t a, size_t *len);
 PL_EXPORT(void)		PL_register_atom(atom_t a);
 PL_EXPORT(void)		PL_unregister_atom(atom_t a);
+PL_EXPORT(size_t)	PL_atom_index(atom_t index);
+PL_EXPORT(atom_t)	PL_atom_from_index(size_t a);
 #ifdef O_DEBUG_ATOMGC
 #define PL_register_atom(a) \
 	_PL_debug_register_atom(a, __FILE__, __LINE__, __PRETTY_FUNCTION__)
@@ -905,7 +908,7 @@ PL_EXPORT(int)		PL_set_prolog_flag(const char *name, int type, ...);
 
 PL_EXPORT(PL_atomic_t)	_PL_get_atomic(term_t t);
 PL_EXPORT(void)		_PL_put_atomic(term_t t, PL_atomic_t a);
-PL_EXPORT(int)		_PL_unify_atomic(term_t t, PL_atomic_t a);
+PL_EXPORT(int)		PL_unify_atomic(term_t t, PL_atomic_t a);
 PL_EXPORT(int)		_PL_get_arg_sz(size_t index, term_t t, term_t a);
 PL_EXPORT(int)		_PL_get_arg(int index, term_t t, term_t a);
 
@@ -1345,24 +1348,26 @@ PL_EXPORT(int)		PL_destroy_engine(PL_engine_t engine);
 		 /*******************************
 		 *	    HASH TABLES		*
 		 *******************************/
-typedef _PLS(table)	  *hash_table_t;
+typedef _PLS(table)	 *hash_table_t;
 typedef _PLS(table_enum) *hash_table_enum_t;
+typedef uint64_t table_key_t;
+typedef uint64_t table_value_t;
 
 #define PL_HT_NEW	0x0001
 #define PL_HT_UPDATE	0x0002
 
-PL_EXPORT(hash_table_t)	PL_new_hash_table(int size,
-					  void (*free_symbol)(void *n, void *v));
+PL_EXPORT(hash_table_t)	PL_new_hash_table(size_t size,
+					  void (*free_symbol)(table_key_t n, table_value_t v));
 PL_EXPORT(int)		PL_free_hash_table(hash_table_t table);
-PL_EXPORT(void*)	PL_lookup_hash_table(hash_table_t table, void *key);
-PL_EXPORT(void*)	PL_add_hash_table(hash_table_t table,
-					  void *key, void *value, int flags);
-PL_EXPORT(void*)	PL_del_hash_table(hash_table_t table, void *key);
+PL_EXPORT(table_value_t) PL_lookup_hash_table(hash_table_t table, table_key_t key);
+PL_EXPORT(table_value_t) PL_add_hash_table(hash_table_t table,
+					   table_key_t key, table_value_t value, int flags);
+PL_EXPORT(table_value_t) PL_del_hash_table(hash_table_t table, table_key_t key);
 PL_EXPORT(int)		PL_clear_hash_table(hash_table_t table);
 PL_EXPORT(hash_table_enum_t) PL_new_hash_table_enum(hash_table_t table);
 PL_EXPORT(void)		PL_free_hash_table_enum(hash_table_enum_t e);
 PL_EXPORT(int)		PL_advance_hash_table_enum(hash_table_enum_t e,
-						   void **key, void **value);
+						   table_key_t *key, table_value_t *value);
 
 
 		 /*******************************
