@@ -49,8 +49,10 @@ test_pipe :-
                  sto(rational_trees)    % rational_trees: only run once
                ]).
 
+wine :-
+    current_prolog_flag(wine_version, _).
 
-test(pwd) :-
+test(pwd, [condition(not(wine))]) :-
     (   current_prolog_flag(windows, true)
     ->  Command = 'cmd /c cd'
     ;   Command = pwd
@@ -95,7 +97,9 @@ test(cat1) :-
     atom_codes(A, String),
     format(atom(A), '~w~n', [Text]).
 :- endif.
-test(cat2, error(io_error(write, _))) :-
+test(cat2, [error(io_error(write, _)),
+	    condition(not(wine))
+	   ]) :-
     (   current_prolog_flag(windows, true)
     ->  Cmd = 'cmd /c rem true',
         Cleanup = true
