@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1996-2021, University of Amsterdam
+    Copyright (c)  1996-2023, University of Amsterdam
                               VU University Amsterdam
 			      CWI, Amsterdam
 			      SWI-Prolog Solutions b.v.
@@ -94,6 +94,7 @@
 #define	PL_get_functor(t, f)			LDFUNC(PL_get_functor, t, f)
 #define	PL_get_name_arity_sz(t, name, arity)	LDFUNC(PL_get_name_arity_sz, t, name, arity)
 #define	PL_unify_atom(t, a)			LDFUNC(PL_unify_atom, t, a)
+#define	PL_unify_atomic(t, w)			LDFUNC(PL_unify_atomic, t, w)
 #define	PL_unify_pointer(t, ptr)		LDFUNC(PL_unify_pointer, t, ptr)
 #define	PL_get_list(l, h, t)			LDFUNC(PL_get_list, l, h, t)
 #define	PL_is_atom(t)				LDFUNC(PL_is_atom, t)
@@ -104,7 +105,7 @@
 #define	PL_clearsig(sig)			LDFUNC(PL_clearsig, sig)
 #define	classify_exception(ex)			LDFUNC(classify_exception, ex)
 #define	classify_exception_p(p)			LDFUNC(classify_exception_p, p)
-#define	get_string_text(atom, text)		LDFUNC(get_string_text, atom, text)
+#define	get_string_text(w, text)		LDFUNC(get_string_text, w, text)
 #endif /*USE_LD_MACROS*/
 
 #define LDFUNC_DECLARATIONS
@@ -121,6 +122,7 @@ void		initialiseForeign(int argc, char **argv);
 void		cleanupInitialiseHooks(void);
 void		cleanAbortHooks(PL_local_data_t *ld);
 atom_t		codeToAtom(int code);
+int		atom_to_bool(atom_t a);
 int		PL_unify_term(term_t t, ...);
 int		PL_unify_termv(term_t t, va_list args);
 term_t		pushWordAsTermRef(Word p);
@@ -128,7 +130,7 @@ void		popTermRef(void);
 int		_PL_get_arg(size_t index, term_t t, term_t a);
 term_t		PL_new_term_ref(void);
 term_t		PL_new_term_ref_noshift(void);
-term_t		PL_new_term_refs(int n);
+term_t		PL_new_term_refs(size_t n);
 functor_t	PL_new_functor_sz(atom_t f, size_t arity);
 int		globalizeTermRef(term_t t);
 void		PL_reset_term_refs(term_t r);
@@ -166,6 +168,7 @@ int		PL_get_name_arity_sz(term_t t, atom_t *name,
 					     size_t *arity);
 int		PL_get_uintptr(term_t t, size_t *i);
 int		PL_unify_atom(term_t t, atom_t a);
+int		PL_unify_atomic(term_t t, word a);
 int		PL_unify_pointer(term_t t, void *ptr);
 int		PL_get_list(term_t l, term_t h, term_t t);
 FLI_INLINE int	PL_is_atom(term_t t);
@@ -197,12 +200,11 @@ void		resetForeign(void);
 
 int		get_atom_ptr_text(Atom atom, PL_chars_t *text);
 int		get_atom_text(atom_t atom, PL_chars_t *text);
-int		get_string_text(atom_t atom, PL_chars_t *text);
+int		get_string_text(word w, PL_chars_t *text);
 
 #undef LDFUNC_DECLARATIONS
 
 #define PL_strip_module(q, m, t) PL_strip_module_flags(q, m, t, 0)
-#define _PL_unify_atomic(t, a)	PL_unify_atom(t, a)
 
 		 /*******************************
 		 *	INLINE DEFINITIONS	*
